package cz.muni.fi.pa165.movie;


import cz.muni.fi.pa165.common.BaseMapper;
import cz.muni.fi.pa165.image.ImageDo;
import cz.muni.fi.pa165.image.ImageMapper;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Mapper(uses = ImageMapper.class,
        collectionMappingStrategy = CollectionMappingStrategy.SETTER_PREFERRED)
public interface MovieMapper extends BaseMapper<MovieDo, MovieDto> {

    /**
     * Updates the given {@code movieDo} entity with the values from the given {@code movieDto}.
     * Images are also updated, and new images are created if necessary.
     *
     * @param movieDto the DTO instance to update the entity from
     * @param movieDo the entity to update
     */
    default void updateFromDto(MovieDto movieDto, @MappingTarget MovieDo movieDo) {
        ImageMapper imageMapper = Mappers.getMapper(ImageMapper.class);
        Map<Long, ImageDo> existingImages = movieDo.getImages().stream()
                .collect(Collectors.toMap(ImageDo::getId, Function.identity()));

        movieDo.setTitle(movieDto.getTitle());
        movieDo.setDescription(movieDto.getDescription());
        movieDo.setDirector(movieDto.getDirector());
        movieDo.setGenres(movieDto.getGenres());
        movieDo.setActors(movieDto.getActors());

        List<ImageDo> updatedImages = movieDto.getImages().stream()
                .map(imageDto -> {
                    if (imageDto.getId() != null && existingImages.containsKey(imageDto.getId())) {
                        ImageDo existingImage = existingImages.get(imageDto.getId());
                        imageMapper.updateFromDto(imageDto, existingImage);
                        return existingImage;
                    } else {
                        ImageDo newImage = imageMapper.fromDto(imageDto);
                        newImage.setMovie(movieDo);
                        return newImage;
                    }
                }).collect(Collectors.toList());
        movieDo.setImages(updatedImages);
    }
}
